@@ -35,8 +35,7 @@
 #include <KActionCollection>
 #include <KConfigGroup>
 #include <KLocalizedString>
-#include <kglobalaccel.h>
-#include <kglobalaccel.h>
+#include <KGlobalAccel>
 
 #include <QDBusInterface>
 #include <QDBusPendingCall>
@@ -73,10 +72,6 @@ KeyboardBrightnessControl::KeyboardBrightnessControl(QObject* parent)
     connect(globalAction, SIGNAL(triggered(bool)), SLOT(toggleKeyboardBacklight()));
 }
 
-KeyboardBrightnessControl::~KeyboardBrightnessControl()
-{
-}
-
 void KeyboardBrightnessControl::onProfileUnload()
 {
     //
@@ -103,8 +98,10 @@ void KeyboardBrightnessControl::onProfileLoad()
         // We don't want to change anything here
         qCDebug(POWERDEVIL) << "Not changing keyboard brightness, the current one is lower and the profile is more conservative";
     } else if (m_defaultValue > 0) {
-        QVariantMap args;
-        args["Value"] = QVariant::fromValue((float)m_defaultValue);
+        const int absoluteValue = qRound(m_defaultValue / 100.0 * keyboardBrightnessMax());
+        QVariantMap args{
+            {QStringLiteral("Value"), QVariant::fromValue(absoluteValue)}
+        };
 
         // plugging in/out the AC is always explicit
         if ((m_currentProfile == "AC" && m_lastProfile != "AC") ||
@@ -117,17 +114,12 @@ void KeyboardBrightnessControl::onProfileLoad()
     }
 }
 
-void KeyboardBrightnessControl::triggerImpl(const QVariantMap& args)
+void KeyboardBrightnessControl::triggerImpl(const QVariantMap &args)
 {
-    if (args.contains("Step")) {
-        backend()->setBrightnessStep(args["Step"].toInt(), BackendInterface::Keyboard);
-    } else if ((QMetaType::Type) args["Value"].type() == QMetaType::Int) {
-        backend()->setBrightnessValue(args["Value"].toInt(), BackendInterface::Keyboard);
-    } else {
-        backend()->setBrightness(args["Value"].toFloat(), BackendInterface::Keyboard);
-    }
-    if (args["Explicit"].toBool() && !args["Silent"].toBool()) {
-        BrightnessOSDWidget::show(keyboardBrightness(), BackendInterface::Keyboard);
+    backend()->setBrightness(args.value(QStringLiteral("Value")).toInt(), BackendInterface::Keyboard);
+
+    if (args.value(QStringLiteral("Explicit")).toBool() && !args.value(QStringLiteral("Silent")).toBool()) {
+        BrightnessOSDWidget::show(keyboardBrightnessPercent(), BackendInterface::Keyboard);
     }
 }
 
@@ -151,8 +143,6 @@ bool KeyboardBrightnessControl::loadAction(const KConfigGroup& config)
 
     if (config.hasKey("value")) {
         m_defaultValue = config.readEntry<int>("value", 50);
-    } else {
-        m_defaultValue = -1;
     }
 
     return true;
@@ -161,96 +151,69 @@ bool KeyboardBrightnessControl::loadAction(const KConfigGroup& config)
 void KeyboardBrightnessControl::onBrightnessChangedFromBackend(const BrightnessLogic::BrightnessInfo &info, BackendInterface::BrightnessControlType type)
 {
     if (type == BackendInterface::Keyboard) {
-        int brightness = qRound(info.percentage);
-        Q_EMIT keyboardBrightnessValueChanged(info.value);
-        Q_EMIT keyboardBrightnessStepChanged(info.step);
-        Q_EMIT keyboardBrightnessChanged(brightness);
+        Q_EMIT keyboardBrightnessChanged(info.value);
+        Q_EMIT keyboardBrightnessMaxChanged(info.valueMax);
     }
 }
 
 void KeyboardBrightnessControl::increaseKeyboardBrightness()
 {
     backend()->brightnessKeyPressed(BrightnessLogic::Increase, BackendInterface::Keyboard);
-    BrightnessOSDWidget::show(keyboardBrightness(), BackendInterface::Keyboard);
+    BrightnessOSDWidget::show(keyboardBrightnessPercent(), BackendInterface::Keyboard);
 }
 
 void KeyboardBrightnessControl::decreaseKeyboardBrightness()
 {
     backend()->brightnessKeyPressed(BrightnessLogic::Decrease, BackendInterface::Keyboard);
-    BrightnessOSDWidget::show(keyboardBrightness(), BackendInterface::Keyboard);
+    BrightnessOSDWidget::show(keyboardBrightnessPercent(), BackendInterface::Keyboard);
 }
 
 void KeyboardBrightnessControl::toggleKeyboardBacklight()
 {
     backend()->brightnessKeyPressed(BrightnessLogic::Toggle, BackendInterface::Keyboard);
-    BrightnessOSDWidget::show(keyboardBrightness(), BackendInterface::Keyboard);
+    BrightnessOSDWidget::show(keyboardBrightnessPercent(), BackendInterface::Keyboard);
 }
 
 int KeyboardBrightnessControl::keyboardBrightness() const
 {
-    return qRound(backend()->brightness(BackendInterface::Keyboard));
+    return backend()->brightness(BackendInterface::Keyboard);
+}
+
+int KeyboardBrightnessControl::keyboardBrightnessMax() const
+{
+    return backend()->brightnessMax(BackendInterface::Keyboard);
 }
 
 void KeyboardBrightnessControl::setKeyboardBrightness(int percent)
 {
-    QVariantMap args;
-    args["Value"] = QVariant::fromValue<float>((float)percent);
-    args["Explicit"] = true;
-    trigger(args);
+    trigger({
+        {QStringLiteral("Value"), QVariant::fromValue(percent)},
+        {QStringLiteral("Explicit"), true}
+    });
 }
 
 void KeyboardBrightnessControl::setKeyboardBrightnessSilent(int percent)
 {
-    QVariantMap args;
-    args["Value"] = QVariant::fromValue<float>((float)percent);
-    args["Explicit"] = true;
-    args["Silent"] = true;
-    trigger(args);
+    trigger({
+        {QStringLiteral("Value"), QVariant::fromValue(percent)},
+        {QStringLiteral("Explicit"), true},
+        {QStringLiteral("Silent"), true}
+    });
 }
 
-int KeyboardBrightnessControl::keyboardBrightnessValue() const
+int KeyboardBrightnessControl::keyboardBrightnessSteps() const
 {
-    return backend()->brightnessValue(BackendInterface::Keyboard);
+    return backend()->brightnessSteps(BackendInterface::Keyboard);
 }
 
-int KeyboardBrightnessControl::keyboardBrightnessValueMax() const
+int KeyboardBrightnessControl::keyboardBrightnessPercent() const
 {
-    return backend()->brightnessValueMax(BackendInterface::Keyboard);
-}
+    const float maxBrightness = keyboardBrightnessMax();
+    if (maxBrightness <= 0) {
+        return 0;
+    }
 
-void KeyboardBrightnessControl::setKeyboardBrightnessValue(int value)
-{
-    QVariantMap args;
-    args["Value"] = QVariant::fromValue<int>(value);
-    args["Explicit"] = true;
-    trigger(args);
-}
-
-void KeyboardBrightnessControl::setKeyboardBrightnessValueSilent(int value)
-{
-    QVariantMap args;
-    args["Value"] = QVariant::fromValue<int>(value);
-    args["Explicit"] = true;
-    args["Silent"] = true;
-    trigger(args);
-}
-
-int KeyboardBrightnessControl::keyboardBrightnessStep() const
-{
-    return backend()->brightnessStep(BackendInterface::Keyboard);
-}
-
-int KeyboardBrightnessControl::keyboardBrightnessStepMax() const
-{
-    return backend()->brightnessStepMax(BackendInterface::Keyboard);
-}
-
-void KeyboardBrightnessControl::setKeyboardBrightnessStep(int step)
-{
-    QVariantMap args;
-    args["Step"] = QVariant::fromValue<int>(step);
-    args["Explicit"] = true;
-    trigger(args);
+    return qRound(keyboardBrightness() / maxBrightness * 100);
 }
 
 }
