@@ -23,8 +23,8 @@
 
 #include "powerdevilbackendinterface.h"
 
-#include <QtCore/QWeakPointer>
-#include <QtCore/QStringList>
+#include <QWeakPointer>
+#include <QStringList>
 
 #include <KSharedConfig>
 
@@ -32,7 +32,6 @@ namespace KActivities
 {
     class Consumer;
 } // namespace KActivities
-typedef QMap< QString, QString > StringStringMap;
 
 class KDirWatch;
 class QDBusServiceWatcher;
@@ -79,6 +78,7 @@ public Q_SLOTS:
 
     QString checkBatteryStatus(bool notify = true);
 
+    QString currentProfile() const;
     void loadProfile(bool force = false);
 
     qulonglong batteryRemainingTime() const;
@@ -120,7 +120,7 @@ private:
 
     QTimer *m_criticalBatteryTimer;
 
-//     KActivities::Consumer *m_activityConsumer;
+    KActivities::Consumer *m_activityConsumer;
 
     // Idle time management
     QHash< Action*, QList< int > > m_registeredActionTimeouts;
@@ -149,6 +149,4 @@ private Q_SLOTS:
 
 }
 
-Q_DECLARE_METATYPE(StringStringMap)
-
 #endif // POWERDEVILCORE_H
